package com.knowbox.audio.sound;

import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Environment;

import java.io.OutputStream;

public class AudioModel {
  final public static String DEFAULT_FILE_LOCATION = Environment.getExternalStorageDirectory().getPath() + "/temp.mp3";
  public int subsDurationMillis = 10;
  public long peakLevelUpdateMillis = 800;
  public boolean shouldProcessDbLevel = true;

  private int mMinBufferSize;
  private AudioRecord mRecorder;
  private boolean mIsRecording = false;

  private MediaRecorder mediaRecorder;
  private Runnable recorderTicker;
  private Runnable dbLevelTicker;
  private long recordTime = 0;
  public final double micLevelBase = 2700;

  private MediaPlayer mediaPlayer;
  private long playTime = 0;

  public MediaRecorder getMediaRecorder() {
    return mediaRecorder;
  }

  public void setMediaRecorder(MediaRecorder mediaRecorder) {
    this.mediaRecorder = mediaRecorder;
  }

  public Runnable getRecorderTicker() {
    return recorderTicker;
  }

  public void setRecorderTicker(Runnable recorderTicker) {
    this.recorderTicker = recorderTicker;
  }

  public Runnable getDbLevelTicker() {
    return dbLevelTicker;
  }

  public void setDbLevelTicker(Runnable ticker){
    this.dbLevelTicker = ticker;
  }

  public long getRecordTime() {
    return recordTime;
  }

  public void setRecordTime(long recordTime) {
    this.recordTime = recordTime;
  }

  public MediaPlayer getMediaPlayer() {
    return mediaPlayer;
  }

  public void setMediaPlayer(MediaPlayer mediaPlayer) {
    this.mediaPlayer = mediaPlayer;
  }

  public long getPlayTime() {
    return playTime;
  }

  public void setPlayTime(long playTime) {
    this.playTime = playTime;
  }

  public boolean isRecording() {
    return mIsRecording;
  }

  public void setRecording(boolean recording) {
    this.mIsRecording = recording;
  }


  public void initRecord(int sampleRate) {
    mMinBufferSize = AudioRecord.getMinBufferSize(sampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);
    mRecorder = new AudioRecord(MediaRecorder.AudioSource.MIC, sampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT, mMinBufferSize * 3);
  }

  public void startRecord(OutputStream fos) throws Exception {
    if (mRecorder.getState() != AudioRecord.STATE_INITIALIZED) {
      throw new Exception("初始化失败");
    }
    mRecorder.startRecording();
    byte[] byteTemp = new byte[mMinBufferSize];
    while (mIsRecording) {
      if (null != mRecorder) {
        int end = mRecorder.read(byteTemp, 0, byteTemp.length);
        fos.write(byteTemp, 0, end);
        fos.flush();
      }
    }
  }

  public void wavStopRecord() {
    try {
      mIsRecording = false;
      if (mRecorder != null) {
        mRecorder.stop();
        mRecorder = null;
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}